export const description =
  'On this page, we’ll dive into the different conversation endpoints you can use to manage conversations programmatically.'


## WebSockets {{ tag: 'GET', label: '/hello_world' }}

<Row>
<Col>
To handle real-time communication, Batman learned how to work with WebSockets. He created a WebSocket class and wrapped it around his Robyn app:
</Col>
  <Col sticky>

    <CodeGroup title="Request" tag="GET" label="/hello_world">

    ```python {{ title: 'untyped' }}
    from robyn import Robyn, jsonify, WebSocket

    app = Robyn(__file__)
    websocket = WebSocket(app, "/web_socket")

    @websocket.on("message")
    def connect():
        return "Hello world, from ws"

    @websocket.on("close")
    def close():
        return "Goodbye world, from ws"

    @websocket.on("connect")
    def message():
        return "Connected to ws"


    ```

    ```python {{title: 'typed'}}
    from robyn import Robyn, jsonify, WebSocket

    app = Robyn(__file__)
    websocket = WebSocket(app, "/web_socket")

    @websocket.on("message")
    def connect():
        return "Hello world, from ws"

    @websocket.on("close")
    def close():
        return "Goodbye world, from ws"

    @websocket.on("connect")
    def message():
        return "Connected to ws"


    ```
    </CodeGroup>
  </Col>
</Row>
<Row>

  <Col>
    For sending a message to the client, Batman used the `sync_send_to` method. 
  </Col>
  <Col sticky>
    <CodeGroup title="Request" tag="GET" label="/hello_world">

      ```python {{ title: 'untyped' }}

        @websocket.on("message")
        def message(ws, msg, global_dependencies) -> str:
            websocket_id = ws.id
            ws.sync_send_to(websocket_id, "This is a message to self")
            return ""

      ```

      ```python {{title: 'typed'}}

        @websocket.on("message")
        def message(ws: WebSocketConnector, msg: str, global_dependencies) -> str:
            websocket_id = ws.id
            state = websocket_state[websocket_id]
            ws.sync_send_to(websocket_id, "This is a message to self")
            return ""

      ```
    </CodeGroup>
  </Col>
</Row>


<Row>

  <Col>
    For sending a message to the client in async manner, Batman used the `async_send_to` method. 
  </Col>
  <Col sticky>
    <CodeGroup title="Request" tag="GET" label="/hello_world">

      ```python {{ title: 'untyped' }}

        @websocket.on("message")
        async def message(ws, msg, global_dependencies) -> str:
            websocket_id = ws.id
            await ws.async_send_to(websocket_id, "This is a message to self")
            return ""

      ```

      ```python {{title: 'typed'}}

        @websocket.on("message")
        async def message(ws: WebSocketConnector, msg: str, global_dependencies) -> str:
            websocket_id = ws.id
            state = websocket_state[websocket_id]
            await ws.async_send_to(websocket_id, "This is a message to self")
            return ""

      ```
    </CodeGroup>
  </Col>
</Row>


<Row>
  <Col>
    For sending broadcast messages, Batman used the `sync_broadcast` method.
  </Col>
  <Col sticky>
    <CodeGroup title="Request" tag="GET" label="/hello_world">

      ```python {{ title: 'untyped' }}

        @websocket.on("message")
        def message(ws, msg, global_dependencies) -> str:
            websocket_id = ws.id
            ws.sync_broadcast("This is a message to self")
            return ""

      ```

      ```python {{title: 'typed'}}

        @websocket.on("message")
        def message(ws: WebSocketConnector, msg: str, global_dependencies) -> str:
            websocket_id = ws.id
            ws.sync_broadcast("This is a message to self")
            return ""

      ```
    </CodeGroup>
  </Col>
</Row>

<Row>
  <Col>
    For sending broadcast messages in async style, Batman used the `async_broadcast` method.
  </Col>
  <Col sticky>
    <CodeGroup title="Request" tag="GET" label="/hello_world">

      ```python {{ title: 'untyped' }}

        @websocket.on("message")
        async def message(ws, msg, global_dependencies) -> str:
            websocket_id = ws.id
            await ws.async_broadcast("This is a message to self")
            return ""

      ```

      ```python {{title: 'typed'}}

        @websocket.on("message")
        async def message(ws: WebSocketConnector, msg: str, global_dependencies) -> str:
            websocket_id = ws.id
            await ws.async_broadcast("This is a message to self")
            return ""

      ```
    </CodeGroup>
  </Col>
</Row>

<Row>
  <Col>
    Robyn also showed Batman to work with query params.
  </Col>
  <Col sticky>
    <CodeGroup title="Request" tag="GET" label="/hello_world?name=gordon&desg=commissioner">

      ```python {{ title: 'untyped' }}

        @websocket.on("message")
        async def message(ws, msg, global_dependencies) -> str:
            websocket_id = ws.id
            if (ws.query_params.get("name") == "gordon" and ws.query_params.get("desg") == "commissioner"):
              ws.sync_broadcast("Gordon authorized to login!")
            return ""

      ```

      ```python {{title: 'typed'}}

        @websocket.on("message")
        async def message(ws: WebSocketConnector, msg: str, global_dependencies) -> str:
            websocket_id = ws.id
            if (ws.query_params.get("name") == "gordon" and ws.query_params.get("desg") == "commissioner"):
              ws.sync_broadcast("Gordon authorized to login!")
            return ""

      ```
    </CodeGroup>
  </Col>
</Row>

---

## What's next?

As the codebase grew, Batman wanted to onboard the justice league to help him manage the application. 

Robyn told him about the different ways he could scale his application, and how to use views and subrouters to make his code more readable. 


- [Views and SubRouters](/documentation/api_reference/views)




